@@ -37,11 +37,31 @@ extern "C" {
  * ever include socket.h.
  */
 enum {
-	AST_AF_UNSPEC	= 0,
-	AST_AF_INET	= 2,
-	AST_AF_INET6	= 10,
+	AST_AF_UNSPEC = AF_UNSPEC,
+	AST_AF_INET   = AF_INET,
+	AST_AF_INET6  = AF_INET6,
 };
 
+enum ast_transport {
+	AST_TRANSPORT_UDP   = 1,
+	AST_TRANSPORT_TCP   = 1 << 1,
+	AST_TRANSPORT_TLS   = 1 << 2,
+	AST_TRANSPORT_WS    = 1 << 3,
+	AST_TRANSPORT_WSS   = 1 << 4,
+};
+
+/*!
+ * \brief
+ * Isolate a 32-bit section of an IPv6 address
+ *
+ * An IPv6 address can be divided into 4 32-bit chunks. This gives
+ * easy access to one of these chunks.
+ *
+ * \param sin6 A pointer to a struct sockaddr_in6
+ * \param index Which 32-bit chunk to operate on. Must be in the range 0-3.
+ */
+#define V6_WORD(sin6, index) ((uint32_t *)&((sin6)->sin6_addr))[(index)]
+
 /*!
  * \brief Socket address structure.
  *
@@ -73,7 +93,7 @@ struct ast_sockaddr {
  * if you know what you're doing.
  *
  * \param addr The IPv4-mapped address to convert
- * \param mapped_addr The resulting IPv4 address
+ * \param ast_mapped The resulting IPv4 address
  * \retval 0 Unable to make the conversion
  * \retval 1 Successful conversion
  */
@@ -242,6 +262,16 @@ static inline char *ast_sockaddr_stringify_addr(const struct ast_sockaddr *addr)
 	return ast_sockaddr_stringify_fmt(addr, AST_SOCKADDR_STR_ADDR);
 }
 
+/*!
+ * \since 12.4
+ *
+ * \brief
+ * Count the 1 bits in a netmask
+ *
+ * \return number of 1 bits
+ */
+int ast_sockaddr_cidr_bits(const struct ast_sockaddr *sa);
+
 /*!
  * \since 1.8
  *
@@ -310,10 +340,10 @@ static inline char *ast_sockaddr_stringify_port(const struct ast_sockaddr *addr)
  * \brief
  * Splits a string into its host and port components
  *
- * \param str[in]   The string to parse. May be modified by writing a NUL at the end of
+ * \param[in] str The string to parse. May be modified by writing a NUL at the end of
  *                  the host part.
- * \param host[out] Pointer to the host component within \a str.
- * \param port[out] Pointer to the port component within \a str.
+ * \param[out] host Pointer to the host component within \a str.
+ * \param[out] port Pointer to the port component within \a str.
  * \param flags     If set to zero, a port MAY be present. If set to PARSE_PORT_IGNORE, a
  *                  port MAY be present but will be ignored. If set to PARSE_PORT_REQUIRE,
  *                  a port MUST be present. If set to PARSE_PORT_FORBID, a port MUST NOT
@@ -343,7 +373,7 @@ int ast_sockaddr_split_hostport(char *str, char **host, char **port, int flags);
  *
  * Host names are NOT allowed.
  *
- * \param[out] addr The resulting ast_sockaddr. This MAY be NULL from 
+ * \param[out] addr The resulting ast_sockaddr. This MAY be NULL from
  * functions that are performing validity checks only, e.g. ast_parse_arg().
  * \param str The string to parse
  * \param flags If set to zero, a port MAY be present. If set to
@@ -383,7 +413,7 @@ int ast_sockaddr_parse(struct ast_sockaddr *addr, const char *str, int flags);
  * port MUST NOT be present.
  *
  * \param family Only addresses of the given family will be returned. Use 0 or
- * AST_SOCKADDR_UNSPEC to get addresses of all families.
+ * AST_AF_UNSPEC to get addresses of all families.
  *
  * \retval 0 Failure
  * \retval non-zero The number of elements in addrs array.
@@ -391,6 +421,23 @@ int ast_sockaddr_parse(struct ast_sockaddr *addr, const char *str, int flags);
 int ast_sockaddr_resolve(struct ast_sockaddr **addrs, const char *str,
 			 int flags, int family);
 
+/*!
+ * \brief
+ * Apply a netmask to an address and store the result in a separate structure.
+ *
+ * When dealing with IPv6 addresses, one cannot apply a netmask with a simple
+ * logical AND operation.  Futhermore, the incoming address may be an IPv4
+ * address and needs to be mapped properly before attempting to apply a rule.
+ *
+ * \param addr The IP address to apply the mask to.
+ * \param netmask The netmask configured in the host access rule.
+ * \param result The resultant address after applying the netmask to the given address
+ * \retval 0 Successfully applied netmask
+ * \retval -1 Failed to apply netmask
+ */
+int ast_sockaddr_apply_netmask(const struct ast_sockaddr *addr, const struct ast_sockaddr *netmask,
+		struct ast_sockaddr *result);
+
 /*!
  * \since 1.8
  *
@@ -467,7 +514,7 @@ int ast_sockaddr_is_ipv4_mapped(const struct ast_sockaddr *addr);
  * \brief
  * Determine if an IPv4 address is a multicast address
  *
- * \parm addr the address to check
+ * \param addr the address to check
  *
  * This function checks if an address is in the 224.0.0.0/4 network block.
  *
@@ -529,6 +576,17 @@ int ast_sockaddr_is_any(const struct ast_sockaddr *addr);
  */
 int ast_sockaddr_hash(const struct ast_sockaddr *addr);
 
+/*!
+ * \since 12.3
+ *
+ * \brief
+ * Returns a string representation of an ast_transport
+ *
+ * \retval Name of the tranpsort if it is defined
+ * \retval Undefined if the transport is undefined
+ */
+const char *ast_transport2str(enum ast_transport transport);
+
 /*!
  * \since 1.8
  *
@@ -647,9 +705,9 @@ int _ast_sockaddr_to_sin(const struct ast_sockaddr *addr,
 /*!
  * \since 1.8
  *
- * \brief
- * Converts a struct sockaddr_in to a struct ast_sockaddr.
+ * \brief Converts a struct sockaddr_in to a struct ast_sockaddr.
  *
+ * \param addr
  * \param sin The sockaddr_in to convert
  * \return an ast_sockaddr structure
  */
