@@ -20,7 +20,7 @@
 #define _RES_PJSIP_SESSION_H
 
 /* Needed for pj_timer_entry definition */
-#include "pjlib.h"
+#include <pjlib.h>
 #include "asterisk/linkedlists.h"
 /* Needed for AST_MAX_EXTENSION constant */
 #include "asterisk/channel.h"
@@ -83,6 +83,10 @@ struct ast_sip_session_media {
 	int timeout_sched_id;
 	/*! \brief Stream is on hold */
 	unsigned int held:1;
+	/*! \brief Does remote support rtcp_mux */
+	unsigned int remote_rtcp_mux:1;
+	/*! \brief Does remote support ice */
+	unsigned int remote_ice:1;
 	/*! \brief Stream type this session media handles */
 	char stream_type[1];
 };
@@ -151,6 +155,14 @@ struct ast_sip_session {
 	struct ast_sip_aor *aor;
 	/*! From header saved at invite creation */
 	pjsip_fromto_hdr *saved_from_hdr;
+	/*! Whether the end of the session should be deferred */
+	unsigned int defer_end:1;
+	/*! Session end (remote hangup) requested while termination deferred */
+	unsigned int ended_while_deferred:1;
+	/*! DTMF mode to use with this session, from endpoint but can change */
+	enum ast_sip_dtmf_mode dtmf;
+	/*! Initial incoming INVITE Request-URI.  NULL otherwise. */
+	pjsip_uri *request_uri;
 };
 
 typedef int (*ast_sip_session_request_creation_cb)(struct ast_sip_session *session, pjsip_tx_data *tdata);
@@ -203,7 +215,7 @@ struct ast_sip_session_supplement {
 	 * This method will always be called from a SIP servant thread.
 	 */
 	void (*session_begin)(struct ast_sip_session *session);
-	/*! 
+	/*!
 	 * \brief Notification that the session has ended
 	 *
 	 * This method may or may not be called from a SIP servant thread. Do
@@ -233,7 +245,7 @@ struct ast_sip_session_supplement {
 	 * There is no guarantee that a channel will be present on the session when this is called.
 	 */
 	int (*incoming_request)(struct ast_sip_session *session, struct pjsip_rx_data *rdata);
-	/*! 
+	/*!
 	 * \brief Called on an incoming SIP response
 	 * This method is always called from a SIP servant thread.
 	 *
@@ -254,7 +266,7 @@ struct ast_sip_session_supplement {
 	 * This method is always called from a SIP servant thread.
 	 */
 	void (*outgoing_request)(struct ast_sip_session *session, struct pjsip_tx_data *tdata);
-	/*! 
+	/*!
 	 * \brief Called on an outgoing SIP response
 	 * This method is always called from a SIP servant thread.
 	 */
@@ -455,6 +467,10 @@ struct ast_sip_session *ast_sip_session_create_outgoing(struct ast_sip_endpoint
  *
  * \param session The session to terminate
  * \param response The response code to use for termination if possible
+ *
+ * \warning Calling this function MAY cause the last session reference to be
+ * released and the session destructor to be called.  If you need to do something
+ * with session after this call, be sure to bump the ref count before calling terminate.
  */
 void ast_sip_session_terminate(struct ast_sip_session *session, int response);
 
@@ -475,6 +491,13 @@ int ast_sip_session_defer_termination(struct ast_sip_session *session);
  */
 void ast_sip_session_defer_termination_cancel(struct ast_sip_session *session);
 
+/*!
+ * \brief End the session if it had been previously deferred
+ *
+ * \param session The session to end if it had been deferred
+ */
+void ast_sip_session_end_if_deferred(struct ast_sip_session *session);
+
 /*!
  * \brief Register an SDP handler
  *
@@ -522,6 +545,13 @@ int ast_sip_session_register_supplement(struct ast_sip_session_supplement *suppl
  */
 void ast_sip_session_unregister_supplement(struct ast_sip_session_supplement *supplement);
 
+/*!
+ * \brief Add supplements to a SIP session
+ *
+ * \param session The session to initialize
+ */
+int ast_sip_session_add_supplements(struct ast_sip_session *session);
+
 /*!
  * \brief Alternative for ast_datastore_alloc()
  *
@@ -587,7 +617,7 @@ void ast_sip_session_remove_datastore(struct ast_sip_session *session, const cha
  * Note: The on_request_creation callback may or may not be called in the same
  * thread where this function is called. Request creation may need to be delayed
  * due to the current INVITE transaction state.
- * 
+ *
  * \param session The session on which the reinvite will be sent
  * \param on_request_creation Callback called when request is created
  * \param on_sdp_creation Callback called when SDP is created
@@ -604,6 +634,23 @@ int ast_sip_session_refresh(struct ast_sip_session *session,
 		enum ast_sip_session_refresh_method method,
 		int generate_new_sdp);
 
+/*!
+ * \brief Regenerate SDP Answer
+ *
+ * This method is used when an SDP offer has been received but an SDP answer
+ * has not been sent yet. It requests that a new local SDP be created and
+ * set as the SDP answer. As with any outgoing request in res_pjsip_session,
+ * this will call into registered supplements in case they wish to add anything.
+ *
+ * \param session The session on which the answer will be updated
+ * \param on_sdp_creation Callback called when SDP is created
+ * \param generate_new_sdp Boolean to indicate if a new SDP should be created
+ * \retval 0 Successfully updated the SDP answer
+ * \retval -1 Failure to updated the SDP answer
+ */
+int ast_sip_session_regenerate_answer(struct ast_sip_session *session,
+		ast_sip_session_sdp_creation_cb on_sdp_creation);
+
 /*!
  * \brief Send a SIP response
  *
